package
{
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.display.Stage;
	import flash.display.StageDisplayState;
	import flash.events.Event;
	import flash.events.MouseEvent;
	
	import flashx.textLayout.formats.VerticalAlign;
	
	import org.osmf.containers.MediaContainer;
	import org.osmf.events.DisplayObjectEvent;
	import org.osmf.events.MediaPlayerStateChangeEvent;
	import org.osmf.events.TimeEvent;
	import org.osmf.layout.HorizontalAlign;
	import org.osmf.layout.LayoutMode;
	import org.osmf.media.MediaPlayer;
	import org.osmf.traits.PlayState;
	
	import uk.msfx.utils.tracing.fp10.Tr;
	import uk.msfx.utils.tracing.users.TrUsers;

	public class ControlBar extends Sprite
	{
		private var _main:Player;
		private var _videoPlayer:MediaPlayer;
		private var _mediaContainer:MediaContainer;
		private var _stage:Stage;
		
		private var _controlBar:Sprite;
		private var _controlBG:ControlBG;
		private var _pauseButton:PauseButton;
		private var _muteButton:MuteButton;
		private var _playToggle:Sprite;
		private var _unmuteButton:UnmuteButton;
		private var _playButton:PlayButton;
		private var _resizeButton:ResizeButton;
		private var _progressBG:ProgressBG;
		private var _progressCurrent:ProgressCurrent;
		private var _mediaCompleted:Boolean;
		
		private var _playerBg:Shape;
		
		public function ControlBar()
		{
		}
		
		public function init(main:Player):void {
			_main 			= main
			_videoPlayer 	= main.videoPlayer;
			_mediaContainer = main.mediaContainer;
			_stage 			= main.mediaContainer.stage;
			
			buildControlBar();
		}
		
		private function buildControlBar():void {
			
			//the player screen outline
			_playerBg = new Shape();
			_playerBg.graphics.lineStyle(1,0x000000);
			_playerBg.graphics.drawRect(0, 0, 480,270);
			
			_controlBar = new Sprite();
			_controlBar.visible = false;
			_controlBar.x = 0;
			_controlBar.y = 271;
			
			_controlBG = new ControlBG();
			
			_pauseButton = new PauseButton();
			_pauseButton.visible = false;
			_pauseButton.addEventListener(MouseEvent.CLICK, onPauseClick);
			
			_playButton = new PlayButton();
			_playButton.visible = true;
			_playButton.addEventListener(MouseEvent.CLICK, onPlayClick);
			
			_playToggle = new Sprite();
			_playToggle.y = 2;
			_playToggle.useHandCursor = true;
			_playToggle.buttonMode = true;
			
			_playToggle.addChild(_playButton);
			_playToggle.addChild(_pauseButton);
			
			_muteButton = new MuteButton();
			_muteButton.y = 2;
			_muteButton.x = 43;
			_muteButton.useHandCursor = true;
			_muteButton.buttonMode = true;
			_muteButton.visible = true;
			_muteButton.addEventListener(MouseEvent.CLICK, onMute);
			
			_unmuteButton = new UnmuteButton();
			_unmuteButton.y = 2;
			_unmuteButton.x = 43;
			_unmuteButton.useHandCursor = true;
			_unmuteButton.buttonMode = true;
			_unmuteButton.visible = false;
			_unmuteButton.addEventListener(MouseEvent.CLICK, onUnmute);
			
			_resizeButton = new ResizeButton();
			_resizeButton.y = 2;
			_resizeButton.x = 86;
			_resizeButton.useHandCursor = true;
			_resizeButton.buttonMode = true;
			_resizeButton.addEventListener(MouseEvent.CLICK, onResize);
			
			_progressBG = new ProgressBG();
			_progressBG.y = 2;
			_progressBG.x = 130;
			_progressBG.useHandCursor = true;
			_progressBG.buttonMode = true;
			_progressBG.addEventListener(MouseEvent.CLICK, onSeek);
			
			_progressCurrent = new ProgressCurrent();
			_progressCurrent.y = 2;
			_progressCurrent.x = 130;
			_progressCurrent.mouseEnabled = false;
			_progressCurrent.scaleX = 0;
			
			addChild(_playerBg);
			addChild(_controlBar);
			
			_controlBar.addChild(_controlBG);
			_controlBar.addChild(_playToggle);
			_controlBar.addChild(_resizeButton);
			_controlBar.addChild(_progressBG);
			_controlBar.addChild(_progressCurrent);
			_controlBar.addChild(_muteButton);
			_controlBar.addChild(_unmuteButton);
			
			/*
				unlock the player controls only after videoplaza load fail or succeed, that will
				stop the user trying to start the player before the SDK is ready
			*/
			_videoPlayer.addEventListener('videoplazaReady',displayControlbar);
			_videoPlayer.addEventListener('videoplazaFail',displayControlbar);
			
			_videoPlayer.addEventListener(TimeEvent.COMPLETE, timeEventHandler);
			_videoPlayer.addEventListener(TimeEvent.CURRENT_TIME_CHANGE, timeEventHandler);
			_videoPlayer.addEventListener('enableControlbar',enableControlBar);
			_videoPlayer.addEventListener('disableControlbar',disableControlbar);
						
			_stage.addEventListener(Event.FULLSCREEN, onEscFullScreen);
		}
		
		private function timeEventHandler(e:TimeEvent):void {
			
			switch(e.type) {
				case TimeEvent.CURRENT_TIME_CHANGE:
					_progressCurrent.scaleX = e.time / _videoPlayer.duration;
					break;
				case TimeEvent.COMPLETE:
					_mediaCompleted = true;
					break;
			}
		}
		
		private function playBtnStateMachine(state:String):void {
			Tr.ace("Controlbar Play toggle state: " + state, TrUsers.CONTROLBAR, ControlBar);
			
			switch(state) 
			{
				case 'playing':
					_playButton.visible 		= false;
					_pauseButton.visible 		= true;
					break;
				case 'paused':
					_playButton.visible 		= true;
					_pauseButton.visible 		= false;
					break;
				case 'enabled':
					_playToggle.visible 		= true;
					_progressCurrent.visible 	= true;
					
					if (_mediaCompleted) {
						_mediaCompleted = false;
						_playButton.visible 	= true;
						_pauseButton.visible 	= false;
					}
					else {
						_playButton.visible 	= false;
						_pauseButton.visible 	= true;
					}
					break;
				case 'disabled':
					_playToggle.visible 		= false;
					_progressCurrent.visible 	= false;
					break;
			}
		}
		
		/*
			When an ad is being displayed the player should disable the controlbar
		*/
		private function disableControlbar(e:Event):void {
			playBtnStateMachine('disabled');
		}
		
		/*
			When an ad has completed being displayed the player should re-enable the controlbar
		*/
		private function enableControlBar(e:Event):void {
			playBtnStateMachine('enabled');
		}
		
		private function onPauseClick(e:MouseEvent):void {
			Tr.ace("User click: Pause", TrUsers.CONTROLBAR, ControlBar);
			
			_videoPlayer.pause();
			playBtnStateMachine('paused');
		}
		
		private function onPlayClick(e:MouseEvent):void {
			Tr.ace("User click: Play", TrUsers.CONTROLBAR, ControlBar);
			
			_videoPlayer.play();
			playBtnStateMachine('playing');
		}
		
		private function onMute(e:MouseEvent):void {
			Tr.ace("User click: Mute", TrUsers.CONTROLBAR, ControlBar);
			
			_videoPlayer.volume = 0;
			_muteButton.visible = false;
			_unmuteButton.visible = true;
		}
		
		private function onUnmute(e:MouseEvent):void {
			Tr.ace("User click: Unmute", TrUsers.CONTROLBAR, ControlBar);
			
			_videoPlayer.volume = 1;
			_muteButton.visible = true;
			_unmuteButton.visible = false;
		}
			
		private function onSeek(e:MouseEvent):void {
			Tr.ace("User click: Seek", TrUsers.CONTROLBAR, ControlBar);
			
			var seekTo:Number = _videoPlayer.duration * (e.target.mouseX/e.target.width);
			_videoPlayer.seek(seekTo);
		}
		
		private function displayControlbar(e:Event):void {
			Tr.ace("Ad Player display controlbar", TrUsers.CONTROLBAR, ControlBar);
			_controlBar.visible = true;
		}
		
		private function onEscFullScreen(e:Event):void {
			Tr.ace("User press scape: Exit fullscreen", TrUsers.CONTROLBAR, ControlBar);
			
			if (_stage.displayState == StageDisplayState.NORMAL) {
				adjustUI('normal');
			}
		}
		
		private function onResize(e:MouseEvent):void {
			Tr.ace("User click: Resize", TrUsers.CONTROLBAR, ControlBar);
			
			if (_stage.displayState == StageDisplayState.NORMAL) {
				_stage.displayState = 'fullScreen';
				adjustUI('fullScreen');		
			}
			else {
				_stage.displayState = 'normal'
				adjustUI('normal');
			}			
		}
		
		//adjust layout elements
		private function adjustUI(format:String):void {
			
			if (format == 'normal') {
				_controlBar.y = 271;
				_main.console.show();
				_playerBg.visible = true;
				
				_mediaContainer.width = 480;
				_mediaContainer.height  = 270;
			}
			else {
				_controlBar.y = _stage.stageHeight - 49;
				_main.console.hide();
				_playerBg.visible = false;
				
				_mediaContainer.width 	= _stage.stageWidth;
				_mediaContainer.height 	= _stage.stageHeight;
			}
		}
	}
}