/**
 * Videoplaza SDK demo integration
 * @author Nelson Diotto
 * @date 2012.11.20
 *  
 *
 * This file is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even 
 * the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * YOU ARE RESPONSIBLE FOR TESTING AND DEBUGGING YOUR IMPLEMENTATION. THIS CLASS and ALL code inside 
 * the folder com.videoplaza WILL NOT BE MAINTAINED BY VIDEOPLAZA, THIS FILE ONLY PURPOSE IS TO BE USE 
 * AS AN EXAMPLE INTEGRATION BETWEEN VIDEOPLAZA SDK AND YOUR PLAYER. THIS CODE IS NOT FIT FOR DEPLOYMENT.
 * 
 **/

package
{
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	
	import org.osmf.containers.MediaContainer;
	import org.osmf.media.DefaultMediaFactory;
	import org.osmf.media.MediaElement;
	import org.osmf.media.MediaFactory;
	import org.osmf.media.MediaPlayer;
	import org.osmf.media.URLResource;
	
	import uk.msfx.utils.tracing.fp10.Tr;
	import uk.msfx.utils.tracing.fp10.TrConsole;
	import uk.msfx.utils.tracing.users.TrUsers;
	
	public class Player extends Sprite
	{
		private static const STREAMING_PATH:String = "rtmp://cp67126.edgefcs.net/ondemand/mp4:mediapm/ovp/content/demo/video/elephants_dream/elephants_dream_768x428_24.0fps_408kbps.mp4";
		
		private var _videoPlayer:MediaPlayer;
		private var _mediaContainer:MediaContainer;
		private var _mediaFactory:DefaultMediaFactory;
		
		private var _controlBar:ControlBar;
		private var _videoplazaPlugin:Videoplaza;
		private var _console:TrConsole;
		
		private var _isAutoplay:Boolean;
		
		public function Player() {
			stage.align 	= StageAlign.TOP_LEFT;
			stage.scaleMode = StageScaleMode.NO_SCALE;
			
			initTracer();
			initPlayer();
		}
		
		private function initPlayer():void {
			Tr.ace("init player", TrUsers.VIDEO_PLAYER, Player);
			
			var resource:URLResource = new URLResource(STREAMING_PATH);
			_mediaFactory = new DefaultMediaFactory();
			
			var element:MediaElement = _mediaFactory.createMediaElement(resource);
			_videoPlayer = new MediaPlayer(element);
			
			_mediaContainer = new MediaContainer();			
			_mediaContainer.addMediaElement(element);
			_mediaContainer.displayObject.width = 480;
			_mediaContainer.displayObject.height  = 270;
						
			addChild(_mediaContainer);
						
			/*
				the video autoplay should be always turned off, instead if autoplay we should make an automatic
				ad request, so if there is a preroll to show it should auto-start
			*/
			_videoPlayer.autoPlay = false; //always false
			
			var flashVars:Object = stage.loaderInfo.parameters;
			if (flashVars.hasOwnProperty('autoPlay')) {
				_isAutoplay = (flashVars.autoPlay == 'true') ? true : false;
			}
																		
			initControlBar();
			initVideoplaza();
		}
		
		/**
		 * The tracer panel, and all classes in the '/uk' folder can be removed.
		 */
		private function initTracer():void {
			_console = new TrConsole(300,270);
			_console.x = 495;			
			
			addChild(console);
		}
		
		/**
		 * Custom controlbar example. Illustrates how to listen and dispatch events
		 * between videoplaza adPlayer and the player controlbar
		 */
		private function initControlBar():void {
			Tr.ace("init controlbar", TrUsers.VIDEO_PLAYER, Player);
			
			_controlBar = new ControlBar();
			_controlBar.init(this);
				
			addChild(_controlBar);
		}
		
		/**
		 * Videoplaza.as encapsulates all logic needed to integrate the Video Player and videoplaza
		 * Ad Player. You can think of Videoplaza.as class as the 'bridge' between the 
		 * Video Player SDK and the Ad Player SDK, the 'bridge' class help translatting events,
		 * methods and states from one framework to the other.
		 */
		private function initVideoplaza():void {
			Tr.ace("init videoplaza", TrUsers.VIDEO_PLAYER, Player);
			
			_videoplazaPlugin = new Videoplaza();
			_videoplazaPlugin.init(this);
				
			addChild(_videoplazaPlugin);
		}
		
		//***************************************************************************************************
		//******************************************* GET/SET ***********************************************
		//***************************************************************************************************
		
		public function get videoplazaPlugin():Videoplaza {
			return _videoplazaPlugin;
		}
		
		public function get controlBar():ControlBar {
			return _controlBar;
		}
		
		public function get console():TrConsole {
			return _console;
		}
		
		public function get videoPlayer():MediaPlayer {
			return _videoPlayer;
		}
		
		public function get mediaContainer():MediaContainer {
			return _mediaContainer;
		}
		
		public function get mediaFactory():MediaFactory {
			return _mediaFactory;
		}
		
		public function get isAutoPlay():Boolean {
			return _isAutoplay;
		}
	}
}