package  {

    import se.videoplaza.kit.api.utils.ItemInfo;
    import com.videoplaza.bridge.core.VideoplazaLoader;
    import com.videoplaza.bridge.core.VpAdPlayer;
    import com.videoplaza.bridge.events.PlayerState;
    import com.videoplaza.bridge.utils.IVideoplazaPlugin;
    
    import flash.display.Sprite;
    import flash.display.Stage;
    import flash.display.StageDisplayState;
    import flash.events.Event;
    
    import org.osmf.containers.MediaContainer;
    import org.osmf.events.AudioEvent;
    import org.osmf.events.MediaPlayerStateChangeEvent;
    import org.osmf.events.TimeEvent;
    import org.osmf.media.MediaPlayer;
    import org.osmf.media.MediaPlayerState;
    import org.osmf.traits.PlayState;
    
    import uk.msfx.utils.tracing.fp10.Tr;
    import uk.msfx.utils.tracing.users.TrUsers;

    public class Videoplaza extends Sprite implements IVideoplazaPlugin {

		private var _main:Player;
		private var _videoPlayer:MediaPlayer;
		private var _mediaContainer:MediaContainer;
		private var _videoPosition:Number;
		private var _vpAdPlayer:VpAdPlayer;
		private var _startNewAdSection:Boolean = true;
		
		private var _stage:Stage;
		private var _overlayModule:Object;
		
		private var _prevHeight:Number = NaN;
		private var _prevWidth:Number= NaN;
		
		public function Videoplaza() {
		}
        
		public function init(main:Player):void {			
			_main 			= main
			_videoPlayer 	= main.videoPlayer;
			_mediaContainer = main.mediaContainer;
			_stage 			= main.mediaContainer.stage;
									
			//load videoplaza SDK from CDN
			var vpLoader:VideoplazaLoader = new VideoplazaLoader();
			vpLoader.init(this);
		}
        
		//***************************************************************************************************
		//************************************* VIDEOLAZA INTERFACE *****************************************
		//***************************************************************************************************
		
		/**
		 * FAIL: Start the videoplayer without videoplaza plugin
		 */
		public function videoplazaLoadedFail(errorMessage:String):void {
			Tr.ace(errorMessage, TrUsers.BRIDGE, Videoplaza);
			
			//tell the player to start witout videoplaza!
			_videoPlayer.dispatchEvent(new Event('videoplazaFail'));
		}
		
		
		/**
		 * SUCCESS: Add videoplaza plugin to the stage
		 */
		public function videoplazaLoadedSuccess(vpAdPlayer:VpAdPlayer):void {
			Tr.ace(" Videoplaza SDK Loaded Success", TrUsers.BRIDGE, Videoplaza);
	
			_vpAdPlayer = vpAdPlayer;
			getStage().addChild(_vpAdPlayer);
						
			setupVideoPlayerListeners();
			
			_prevWidth = _mediaContainer.width;
			_prevHeight = _mediaContainer.height;
			
			//if autoplay, start a new ad section straightaway
			if (_main.isAutoPlay) {
				_startNewAdSection = false;
				setNewItem();
				_vpAdPlayer.setPlayerState(PlayerState.PAUSED);
			}
				
			
			//tell the player videoplaza is ready, now you can start!
			_videoPlayer.dispatchEvent(new Event('videoplazaReady'));
		}
		
		/**
		 * REQUEST NEW AD SECTION: here we passe the configuration for the 
		 * request using the itemInfo object, _vpAdPlayer.setNewItem will trigger
		 * a new HTTP request to videoplaza backend and get a JSON/XML response
		 * containing ads.
		 * 
		 * @param media = a reference to the current media object
		 */
		public function setNewItem(media:Object=null):void {
			
			var videoplazaConfig:Object = _stage.loaderInfo.parameters;
				
			var itemInfo:ItemInfo 		= new ItemInfo();
				itemInfo.category 		= (videoplazaConfig.vpcategory) ? videoplazaConfig.vpcategory : '';
				itemInfo.tags 			= (videoplazaConfig.vptags) ? videoplazaConfig.vptags.split(',') : [];
				itemInfo.flags 			= (videoplazaConfig.vpflags) ? videoplazaConfig.vpflags.split(',') : [];
				itemInfo.contentForm 	= (videoplazaConfig.contentForm) ? videoplazaConfig.contentForm : '';
				itemInfo.contentId 		= (videoplazaConfig.contentId) ? videoplazaConfig.contentId : '';
				itemInfo.contentPartner = (videoplazaConfig.contentPartner) ? videoplazaConfig.contentPartner : '';
				
				//cuepoints must be formated as {name:'vpspot', time:10}
				if (videoplazaConfig.cuepoints) {
					var cuePointMarkers:Array = videoplazaConfig.cuepoints.split(",");
					var formatedMarkers:Array = [];
					for each (var time:String in cuePointMarkers) {
						formatedMarkers.push({name:'vpspot', time:time});
					}
					itemInfo.cuePoints = formatedMarkers;
				}
				
			Tr.ace("*** Start a new Ad Section (setNewItem) ***", TrUsers.BRIDGE, Videoplaza);
			Tr.aceObject(itemInfo, TrUsers.BRIDGE, Videoplaza);
			
			_vpAdPlayer.setNewItem(itemInfo);
		}
		
		/**
		 * Every time a new ad module is created it will fire this method, this can be 
		 * used to add further costumizations and listeners to the ad modules.
		 */
		public function onAdModuleCreated(moduleEvent:Event):void {		
			var adModuleName:String = Object(moduleEvent).getData();
						
			Tr.ace("New Ad module created: " + adModuleName, TrUsers.BRIDGE, Videoplaza);
			
			// add a padding to the bottom of the overlay ad, so it wont overlap the controlbar
			if (adModuleName === "overlay") {
				_overlayModule = adPlayer.getAdModule(adModuleName);
				
				getStage().addEventListener(Event.RESIZE, overlayPadding);
			}	
		}
		
		private function overlayPadding(e:Event):void {
			if (isFullScreen) {
				_overlayModule.setPadding(0,0,0,50);
			}
			else {
				_overlayModule.setPadding(0,0,0,0);
			}
		}
		
		//***************************************************************************************************
		//************************************ PLAYER EVENT LISTENERS ***************************************
		//***************************************************************************************************
		
		private function setupVideoPlayerListeners():void {

			_videoPlayer.addEventListener(AudioEvent.MUTED_CHANGE, audioEventHandler);
			_videoPlayer.addEventListener(AudioEvent.VOLUME_CHANGE, audioEventHandler);
			
			_videoPlayer.addEventListener(MediaPlayerStateChangeEvent.MEDIA_PLAYER_STATE_CHANGE, mediaEventHandler);
						
			_videoPlayer.addEventListener(TimeEvent.COMPLETE, timeEventHandler);
			_videoPlayer.addEventListener(TimeEvent.CURRENT_TIME_CHANGE, timeEventHandler);
			_videoPlayer.addEventListener(TimeEvent.DURATION_CHANGE, timeEventHandler);
			
			getStage().addEventListener(Event.RESIZE, stageEventHandler);
		}
		
		private function stageEventHandler(e:Event):void {
			_vpAdPlayer.setPlayerState(PlayerState.RESIZE);
		}
		
		private function mediaEventHandler(e:MediaPlayerStateChangeEvent):void {
			
			Tr.ace("Video Player Dispach Event: " + e.state, TrUsers.VIDEO_PLAYER, Videoplaza);
			
			switch(e.state) 
			{
				case PlayState.PLAYING:
					if (_startNewAdSection) { //stop multiple ad call requests being made at the same time
						_videoPlayer.stop();
						_startNewAdSection = false;
						setNewItem();
					}
					_vpAdPlayer.setPlayerState(PlayerState.PLAYING);
					break;
				case PlayState.STOPPED:
					_vpAdPlayer.setPlayerState(PlayerState.PAUSED);
					break;
				case PlayState.PAUSED:
					_vpAdPlayer.setPlayerState(PlayerState.PAUSED);
					break;
			}
		}
		
		private function audioEventHandler(e:AudioEvent):void {
			Tr.ace("Video Player Dispach Event: " + e.type, TrUsers.VIDEO_PLAYER, Videoplaza);
			
			_vpAdPlayer.setPlayerState(PlayerState.VOLUME_CHANGE);
		}
		
		private function timeEventHandler(e:TimeEvent):void {
			
			if (e.type !== TimeEvent.CURRENT_TIME_CHANGE) {
				Tr.ace("Video Player Dispach Event: " + e.type, TrUsers.VIDEO_PLAYER, Videoplaza);
			}
			
			switch(e.type) {
				case TimeEvent.CURRENT_TIME_CHANGE:
					_videoPosition = e.time;
					break;
				case TimeEvent.DURATION_CHANGE:
					//tell videoplaza adplayer how long the video is as soon the duration metadata arrives
					var meta:Object = new Object();
					meta.duration =  e.time;					
					if (meta.duration)
						_vpAdPlayer.setMetaData(meta);
					break;
				case TimeEvent.COMPLETE:
					_vpAdPlayer.setPlayerState(PlayerState.CLIP_COMPLETE);
					_startNewAdSection = true;
					break;
			}
		}

		//***************************************************************************************************
		//******************************************* GET/SET ***********************************************
		//***************************************************************************************************
		
		public function get videoplazaConfig():Object {	
			var vpConfig:Object = _mediaContainer.stage.loaderInfo.parameters;
			
			Tr.ace("*** get videoplaza plugin config ***", TrUsers.BRIDGE, Videoplaza);
			Tr.aceObject(vpConfig, TrUsers.BRIDGE, Videoplaza);
			
			return _stage.loaderInfo.parameters;
		}
		
		public function get isMuted():Boolean {
			return _videoPlayer.muted;
		}
		
		public function get isFullScreen():Boolean {
			return (getStage().displayState == StageDisplayState.FULL_SCREEN) ? true : false;
		}
		
		public function getStage():Stage {
			return _mediaContainer.stage;
		}
		
		public function getScreenX():Number {
			return _mediaContainer.x;
		}
		public function getScreenY():Number {
			return _mediaContainer.y;
		}
		public function getScreenWidth():Number {
			var width:Number;
			
			if (isFullScreen)
				width = getStage().fullScreenWidth;
			else if (_prevWidth)
				width = _prevWidth;  //fix for OSMF giving old size values
			else
				width = _mediaContainer.width;
			
			return width;
		}
		public function getScreenHeight():Number{
			var height:Number;
			
			if (isFullScreen)
				height = getStage().fullScreenHeight;
			else if (_prevHeight)
				height = _prevHeight; //fix for OSMF giving old size values
			else
				height = _mediaContainer.height;
			
			return height;
		}
		
		public function getPlayerVolume():Number {
			return _videoPlayer.volume;
		}
		
		public function getVideoPosition():Number {
			return _videoPosition;
		}
		
		public function setScreenPosition(x:Number,y:Number):void {
			_mediaContainer.x = x;
			_mediaContainer.y = y;
		}
		
		public function setScreenSize(w:Number,h:Number):void {
			_mediaContainer.width = w;
			_mediaContainer.height = h;
		}
		
		public function setPlayerVolume(volume:Number):void {
			_videoPlayer.volume = volume;
		}
		
		public function setPlayerPlay():void {
			_videoPlayer.play();
		}
		
		public function setPlayerPause():void {
			_videoPlayer.pause();
		}
		
		public function setPlayerControlsEnable(enable:Boolean):void {
			if (enable) {
				_videoPlayer.dispatchEvent(new Event('enableControlbar'));
			}
			else {
				_videoPlayer.dispatchEvent(new Event('disableControlbar'));
			}		
		}
		
		/**
		 * Expose vp adplayer so other classes in your player can listen or manipulate 
		 * the adplayer object
		 */		
		public function get adPlayer():VpAdPlayer {
			return _vpAdPlayer;
		}
    }
}

