package com.videoplaza.bridge.core
{
	import se.videoplaza.kit.juggernaut.debug.VpDebug;
	import se.videoplaza.kit.api.utils.VideoplazaLoader;
	import se.videoplaza.kit.api.utils.VideoplazaLoaderConfig;
	import com.videoplaza.bridge.utils.IVideoplazaPlugin;
	
	public class VideoplazaLoader extends se.videoplaza.kit.api.utils.VideoplazaLoader
	{				
		private var _vpPlugin:IVideoplazaPlugin;
		private var _vpAdPlayer:VpAdPlayer;
		private var _vpConfig:VideoplazaLoaderConfig;
		private var _content:Object = {};
		
		//***************************************************************************************************
		//************************************** LOAD VIDEOPLAZA ********************************************
		//***************************************************************************************************
		
		public final function init(vpPlugin:IVideoplazaPlugin):void {
			_vpPlugin = vpPlugin;
			
			try {
				loadVideoplaza(getVpConfig(), loadSuccess, loadFail);
			} 
			catch (error:Error) {
				loadFail("fail to find videoplaza loader configuration. " + error.toString());
			}
		}
		
		/**
		 * lowercase vp loader configutration
		 */ 
		private function getVpConfig():VideoplazaLoaderConfig {
			
			var adPlayerConfig:VideoplazaLoaderConfig = new VideoplazaLoaderConfig();
			var pluginConfig:Object = _vpPlugin.videoplazaConfig;
			
			if (!pluginConfig.hasOwnProperty('vpdomain') && !pluginConfig.hasOwnProperty('vphost')) {
				throw new Error("Account not configured, missing one of [vpHost, vpDomain].");
				return;
			}
			
			//replace vphost with vpdomain, until Adplayer add native support
			if (pluginConfig.hasOwnProperty('vphost')) {
				pluginConfig.vpdomain = pluginConfig.vphost;
			}
			
			for (var key:String in pluginConfig) {
				var lcKey:String = key.toLowerCase();
				
				// stop from setting those values twice. Those flashvars will be set in Videoplaza.setNewItem();
				if (lcKey !== 'vptags' && 
					lcKey !== 'vpshares' && 
					lcKey !== 'vpflags' && 
					lcKey !== 'vpcategory' && 
					lcKey !== 'vpcontentpartner' && 
					lcKey !== 'vpcontentform' && 
					lcKey !== 'vpcontentid') {
					adPlayerConfig[lcKey] = pluginConfig[key];
				}
			}
			return adPlayerConfig;
		}
		
		/**
		 * Fail callback function for the Videoplaza loader
		 */ 
		private function loadFail(errorMessage:String="fail to load videoplaza BootLoader."):void {
			_vpPlugin.videoplazaLoadedFail(errorMessage);
		}
		
		/**
		 * Success callback function for the Videoplaza loader
		 */ 
		private function loadSuccess(videoplazaBootLoaderObject:Object):void {
			try {
				var vpObject:Object = videoplazaBootLoaderObject.createAdPlayer({});
				_vpAdPlayer = new VpAdPlayer(vpObject,_vpPlugin); //init adPlayer
			} 
			catch (error:Error) {
				_vpPlugin.videoplazaLoadedFail("fail to initiate videoplaza BootLoader. " + error.toString());
				return;
			}
			
			_vpPlugin.videoplazaLoadedSuccess(_vpAdPlayer); //give to the video player a reference to adPlayer
		}
	}
}