package com.videoplaza.bridge.core
{
	import se.videoplaza.kit.api.VpAdPlayerWrapper;
	import se.videoplaza.kit.api.constants.InsertionPointType;
	import se.videoplaza.kit.api.events.VpCommandEventType;
	import se.videoplaza.kit.api.events.VpEventType;
	import se.videoplaza.kit.api.utils.ItemInfo;
	import se.videoplaza.kit.api.utils.ViewMode;
	import com.videoplaza.bridge.events.PlayerState;
	import com.videoplaza.bridge.utils.IVideoplazaPlugin;
	
	import flash.events.Event;
	
	import uk.msfx.utils.tracing.fp10.Tr;
	import uk.msfx.utils.tracing.users.TrUsers;
	
	public class VpAdPlayer extends VpAdPlayerWrapper
	{
		private var _vpPlugin:IVideoplazaPlugin;
		private var _isAdplayerPausing:Boolean = true;
		
		public function VpAdPlayer(vpObject:Object,vpPlugin:IVideoplazaPlugin) {
			super(vpObject);
			
			_vpPlugin = vpPlugin;
			setupAdPlayerListeners();
			resizeAdPlayer();
			volumeChangeAdPlayer();
		}
		
		//***************************************************************************************************
		//**************************************** SET NEW ITEM *********************************************
		//***************************************************************************************************
		
		override public function setNewItem(itemInfo:ItemInfo, array:Array=null):void {
			
			itemInfo.timeReference = new TimeReference(_vpPlugin);
			
			super.setNewItem(itemInfo);
		}
		
		//***************************************************************************************************
		//************************************* RESIZE AND VOLUME CHANGE ************************************
		//***************************************************************************************************
		
		public override function setPlayerState(state:String):void {
			
			Tr.ace("Ad Player Update Player State: " + state, TrUsers.AD_PlAYER, VpAdPlayer); //can delete
			
			switch (state)
			{
				case "VOLUME_CHANGE":
					volumeChangeAdPlayer();
					break;
				case "RESIZE":
					resizeAdPlayer();
					break;
				case PlayerState.PAUSED:					
					//filter all adplayer pause events, we only want trigger pause ads when the user clicks the pause btn
					if (!_isAdplayerPausing) {
						triggerInsertionPoint(InsertionPointType.ON_PAUSE);
					}
					super.setPlayerState(state);
					break;
				default:
					super.setPlayerState(state);
			}
		}
		
		private function resizeAdPlayer():void {
			x = _vpPlugin.getScreenX();
			y = _vpPlugin.getScreenY();
			
			var sMode:String = _vpPlugin.isFullScreen ? ViewMode.FULLSCREEN : ViewMode.NORMAL;
			setSize(_vpPlugin.getScreenWidth(), _vpPlugin.getScreenHeight(),sMode);
		}
		
		private function volumeChangeAdPlayer():void {
			var volume:Number = (_vpPlugin.isMuted) ? 0 : _vpPlugin.getPlayerVolume();
			setVolume(volume);
		}
		
		//***************************************************************************************************
		//*************************************** EVENT LISTENERS *******************************************
		//***************************************************************************************************
		
		private function setupAdPlayerListeners():void { 
			
			addEventListener(VpEventType.AD_MODULE_CREATED, _vpPlugin.onAdModuleCreated);
			
			addEventListener(VpEventType.AD_PLAYER_READY, adPlayerEventHandler);
			addEventListener(VpEventType.AD_PLAYER_COMPLETE, adPlayerEventHandler);
			addEventListener(VpEventType.AD_PLAYER_VOLUME_CHANGE, adPlayerEventHandler);
			addEventListener(VpEventType.AD_PLAYER_MUTED, adPlayerEventHandler);
			
			addEventListener(VpCommandEventType.RESIZE_CONTENT, adPlayerEventHandler);
			addEventListener(VpCommandEventType.ENABLE_CONTROLS, adPlayerEventHandler);
			addEventListener(VpCommandEventType.DISABLE_CONTROLS, adPlayerEventHandler);
			addEventListener(VpCommandEventType.START, adPlayerEventHandler); 
			addEventListener(VpCommandEventType.PLAY, adPlayerEventHandler); 
			addEventListener(VpCommandEventType.PAUSE, adPlayerEventHandler);
		}
		
		private function adPlayerEventHandler(e:Event):void {			
			Tr.ace("Ad Player Dispach Event: " + e.type, TrUsers.AD_PlAYER, VpAdPlayer); //can delete
			
			switch (e.type)
			{
				case VpEventType.AD_PLAYER_READY:
					start();
					break;
				case VpCommandEventType.START:
					break;
				case VpCommandEventType.DISABLE_CONTROLS:
					_vpPlugin.setPlayerControlsEnable(false);
					break;
				case VpCommandEventType.ENABLE_CONTROLS:
					_vpPlugin.setPlayerControlsEnable(true);
					break;
				case VpCommandEventType.PLAY:
					_vpPlugin.setPlayerPlay();
					_isAdplayerPausing = false;
					break;
				case VpCommandEventType.PAUSE:
					_vpPlugin.setPlayerPause();
					_isAdplayerPausing = true;
					break;
				case VpCommandEventType.RESIZE_CONTENT:
					if (_vpPlugin.isFullScreen) {
						_vpPlugin.setScreenSize(_vpPlugin.getStage().fullScreenWidth, _vpPlugin.getStage().fullScreenHeight);
						_vpPlugin.setScreenPosition(0,0);
					}
					else {
						_vpPlugin.setScreenSize(width,height);
						_vpPlugin.setScreenPosition(x,y);
					}
					break;
				case VpEventType.AD_PLAYER_VOLUME_CHANGE:
					_vpPlugin.setPlayerVolume(getVolume());
					break;
				case VpEventType.AD_PLAYER_MUTED:
					_vpPlugin.setPlayerVolume(getVolume());
					break;
				case VpEventType.AD_PLAYER_COMPLETE:
					_isAdplayerPausing = true;
					reset();
					break;
			}
		}
		
		//***************************************************************************************************
		//******************************************* GET/SET ***********************************************
		//***************************************************************************************************
		
		public function get isMuted():Boolean {
			return (getVolume() == 0) ? true : false;
		}
	}
}